==> JavaScript-QuickJS-0.01_01.tar.gz: JavaScript-QuickJS-0.01_01/easyxs/easyxs_perlcall.h <==
#ifndef EASYXS_PERLCALL_H
#define EASYXS_PERLCALL_H 1

#include "init.h"

static inline void _EASYXS_SET_ARGS (pTHX_ SV* object, SV** args) {
    dSP;

    unsigned argscount = 0;

    if (args) {
        while (args[argscount] != NULL) argscount++;
    }

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

    if (object || argscount) {
        EXTEND(SP, (object ? 1 : 0) + argscount);

        if (object) PUSHs( sv_mortalcopy(object) );

        unsigned a=0;
        while (a < argscount) mPUSHs( args[a++] );

        PUTBACK;
    }
}

#define exs_call_sv_void(sv, args) STMT_START { \
    _EASYXS_SET_ARGS(aTHX_ NULL, args);         \
    call_sv(sv, G_DISCARD | G_VOID);            \
    FREETMPS;                                   \
    LEAVE;                                      \
} STMT_END

#define exs_call_method_void(object, methname, args) STMT_START { \
    _EASYXS_SET_ARGS(aTHX_ object, args);                 \
                                                    \
    call_method( methname, G_DISCARD | G_VOID );    \
                                                    \
    FREETMPS;                                       \
    LEAVE;                                          \
} STMT_END

static inline SV* _easyxs_fetch_scalar_return (pTHX_ int count) {
    dSP;

    SPAGAIN;

    SV* ret;

    if (count == 0) {
        ret = &PL_sv_undef;
    }
    else {
        ret = SvREFCNT_inc(POPs);

        while (count-- > 1) PERL_UNUSED_VAR(POPs);
    }

    PUTBACK;
    FREETMPS;
    LEAVE;

    return ret;
}

static inline SV* _easyxs_call_method_scalar (pTHX_ SV* object, const char* methname, SV** args) {
    _EASYXS_SET_ARGS(aTHX_ object, args);

    int count = call_method(methname, G_SCALAR);

    return _easyxs_fetch_scalar_return(aTHX_ count);
}

#define exs_call_method_scalar(object, methname, args) \
    _easyxs_call_method_scalar(aTHX_ object, methname, args)

static inline SV* _easyxs_call_sv_scalar (pTHX_ SV* cb, SV** args) {
    _EASYXS_SET_ARGS(aTHX_ NULL, args);

    int count = call_sv(cb, G_SCALAR);

    return _easyxs_fetch_scalar_return(aTHX_ count);
}

#define exs_call_sv_scalar(sv, args) \
    _easyxs_call_sv_scalar(aTHX_ sv, args)

#endif

==> JavaScript-QuickJS-0.06.tar.gz: JavaScript-QuickJS-0.06/easyxs/easyxs_perlcall.h <==
#ifndef EASYXS_PERLCALL_H
#define EASYXS_PERLCALL_H 1

#include "init.h"

static inline void _EASYXS_SET_ARGS (pTHX_ SV* object, SV** args) {
    dSP;

    unsigned argscount = 0;

    if (args) {
        while (args[argscount] != NULL) argscount++;
    }

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

    if (object || argscount) {
        EXTEND(SP, (object ? 1 : 0) + argscount);

        if (object) PUSHs( sv_mortalcopy(object) );

        unsigned a=0;
        while (a < argscount) mPUSHs( args[a++] );

        PUTBACK;
    }
}

#define exs_call_sv_void(sv, args) STMT_START { \
    _EASYXS_SET_ARGS(aTHX_ NULL, args);         \
    call_sv(sv, G_DISCARD | G_VOID);            \
    FREETMPS;                                   \
    LEAVE;                                      \
} STMT_END

#define exs_call_method_void(object, methname, args) STMT_START { \
    _EASYXS_SET_ARGS(aTHX_ object, args);                 \
                                                    \
    call_method( methname, G_DISCARD | G_VOID );    \
                                                    \
    FREETMPS;                                       \
    LEAVE;                                          \
} STMT_END

static inline SV* _easyxs_fetch_scalar_return (pTHX_ int count) {
    dSP;

    SPAGAIN;

    SV* ret;

    if (count == 0) {
        ret = &PL_sv_undef;
    }
    else {
        ret = SvREFCNT_inc(POPs);

        while (count-- > 1) PERL_UNUSED_VAR(POPs);
    }

    PUTBACK;
    FREETMPS;
    LEAVE;

    return ret;
}

static inline SV* _easyxs_call_method_scalar (pTHX_ SV* object, const char* methname, SV** args) {
    _EASYXS_SET_ARGS(aTHX_ object, args);

    int count = call_method(methname, G_SCALAR);

    return _easyxs_fetch_scalar_return(aTHX_ count);
}

#define exs_call_method_scalar(object, methname, args) \
    _easyxs_call_method_scalar(aTHX_ object, methname, args)

static inline SV* _easyxs_call_sv_scalar (pTHX_ SV* cb, SV** args) {
    _EASYXS_SET_ARGS(aTHX_ NULL, args);

    int count = call_sv(cb, G_SCALAR);

    return _easyxs_fetch_scalar_return(aTHX_ count);
}

#define exs_call_sv_scalar(sv, args) \
    _easyxs_call_sv_scalar(aTHX_ sv, args)

static inline SV* _easyxs_call_sv_scalar_trapped (pTHX_ SV* cb, SV** args, SV** error) {
    _EASYXS_SET_ARGS(aTHX_ NULL, args);

    int count = call_sv(cb, G_SCALAR | G_EVAL);

    dSP;

    SV* err_tmp = ERRSV;
    if (SvTRUE(err_tmp)) {
        while (count--) PERL_UNUSED_VAR(POPs);

        *error = err_tmp;

        PUTBACK;
        FREETMPS;
        LEAVE;

        return NULL;
    }

    return _easyxs_fetch_scalar_return(aTHX_ count);
}

#define exs_call_sv_scalar_trapped(sv, args, err_p) \
    _easyxs_call_sv_scalar_trapped(aTHX_ sv, args, err_p)

#endif
